package com.iovhm.mock_dify;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

@RequestMapping("/dify")
@RestController
@Slf4j
public class DifyController {

    @Value("${dify-api.url:}")
    private String difyApiUrl;

    @Value("${dify-api.auth-token:}")
    private String difyAuthToken;

    // 在全局访问路径的基础上，成员方法的访问路径
    @RequestMapping("/v1/chat-messages")
    public String hello(@RequestBody(required = false) Map reqBody) {

        // 检查 reqBody 是否为空
        if (reqBody == null) {
            return JSONUtil.createObj()
                    .put("code", 400)
                    .put("message", "请求体不能为空")
                    .toString();
        }

        // 检查 query 参数
        Object queryObj = reqBody.get("query");
        if (queryObj == null || !(queryObj instanceof String) || ((String) queryObj).trim().isEmpty()) {
            return JSONUtil.createObj()
                    .put("code", 400)
                    .put("message", "参数 'query' 不能为空或非字符串类型")
                    .toString();
        }

        log.info("接收到参数: {}", reqBody);

        // 构造请求头
        Map<String, String> headersMap = new HashMap<>();
        headersMap.put("Authorization", difyAuthToken);
        headersMap.put("Content-Type", "application/json");

        // 构造请求体
        Map<String, Object> body = new HashMap<>();
        body.put("inputs", new HashMap<>());
        body.put("query", queryObj);
        body.put("response_mode", "blocking");
        body.put("user", "abc-123");
        body.put("username", "abc-123");

        String jsonBody = JSONUtil.toJsonStr(body);

        HttpResponse response;
        String result2;
        try {
            // 发起请求
            response = HttpRequest.post(difyApiUrl)
                    .headerMap(headersMap, true)
                    .body(jsonBody)
                    .execute();

            result2 = response.body();
            log.info("AI 响应: {}", result2);

            if (!response.isOk()) {
                return JSONUtil.createObj()
                        .put("code", response.getStatus())
                        .put("message", "调用 AI 接口失败: " + response.body())
                        .toString();
            }

        } catch (Exception e) {
            log.error("调用 AI 接口失败: {}", e);
            // 可以记录日志
            return JSONUtil.createObj()
                    .put("code", 500)
                    .put("message", "调用 AI 接口失败: " + e.getMessage())
                    .toString();
        }
        // 解析响应
        JSONObject resultJson;
        try {
            resultJson = JSONUtil.parseObj(result2);
        } catch (Exception e) {
            log.error("AI 响应格式错误: {}", e);
            return JSONUtil.createObj()
                    .put("code", 500)
                    .put("message", "AI 返回的响应格式错误")
                    .toString();
        }

        Object answer = resultJson.containsKey("answer")
                ? resultJson.get("answer")
                : null;
        if (null == answer) {
            log.error("AI 响应缺少 'answer' 字段");
            return JSONUtil.createObj()
                    .put("code", 500)
                    .put("message", "AI 接口返回内容缺失 'answer'")
                    .toString();
        }

        // answer是一个字符串，但也有可能是一段被格式化的json，尝试进行解析
        if (answer instanceof String) {
            try {
                answer = JSONUtil.parse(answer);
            } catch (Exception e) {
                log.info("AI 响应 'answer' 字段内容为 {}", answer);
            }
        }
        // 返回成功响应
        JSONObject resultBody = JSONUtil.createObj();
        resultBody.put("code", 200);
        resultBody.put("message", "success");
        resultBody.put("data", answer);

        return resultBody.toString();
    }
}